package edu.columbia.cs.irt.rfidentify;

import java.io.IOException;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.Properties;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import edu.columbia.cs.irt.rfidentify.core.PropertyLoader;
import edu.columbia.cs.irt.rfidentify.dbhandler.DBAccess;
import edu.columbia.cs.irt.rfidentify.dbhandler.DBInitializer;
import edu.columbia.cs.irt.rfidentify.display.DisplayName;
import edu.columbia.cs.irt.rfidentify.display.IdTagHandler;
import edu.columbia.cs.irt.rfidentify.display.SimplePresenceHandler;
import edu.columbia.cs.irt.rfidentify.display.TextMarksHandler;
import edu.columbia.cs.irt.rfidentify.presence.Presence;
import edu.columbia.cs.irt.rfidentify.presence.SipLayer;
import edu.columbia.cs.irt.rfidentify.registration.RegistrationHandler;
import edu.columbia.cs.irt.rfidentify.server.RequestListenerThread;
import edu.columbia.cs.irt.rfidentify.server.ZeroConfigRegister;

/**
 * Main Startup function for RFidentify
 * @author "Blake Arnold"
 *
 */
class Startup {
    static Logger logger = LoggerFactory.getLogger(Startup.class);

    static DBAccess dbAccess;
    static RequestListenerThread serverThread;

    /**
     * calls init functions
     * @param args
     */
    public static void main(String[] args){
    	initWebServer();
    	initDatabase();
    	initIdTagHandlers();
    	initKiosk();
    	serverThread.start();
    
    }
    
    /**
     * starts up webserver and zeroconfig
     */
    private static void initWebServer(){
    	
		try {
			serverThread = new RequestListenerThread();
			serverThread.setDaemon(false);
			
		} catch (IOException e) {
			logger.error("could not start apache webserver", e);
			logger.error("Could not bind port " + e.getMessage(), e);
			System.exit(1);
		}
		
		try {
	    	ZeroConfigRegister zconfig = new ZeroConfigRegister(serverThread.getPort());
	    	zconfig.start();
		} catch (NoClassDefFoundError e) {
			logger.error("mDNS is not in class path, cannot start bonjour service", e);
		} 
       



    }
    
    /**
     * creates database
     */
    private static void initDatabase(){
    	String pName = "database.properties";
    	String database;
    	boolean overwrite;
			try {
				Properties props = PropertyLoader.loadProps(pName);
				database = props.getProperty("database");
				String mode = props.getProperty("mode");
				overwrite = mode == null ? false : mode.equals("test");
			} catch (IOException e) {
				logger.error("No properties file database.properties, using default",e);
				database = "jdbc:sqlite:rfidentify.db";
				overwrite = false;
			}
			
    	
		try {
			Class.forName("org.sqlite.JDBC");
		} catch (ClassNotFoundException e) {
			logger.error("sqlitejdbc.jar is not in classpath", e);
			System.exit(1);
		} 


	    if(overwrite){
	    	try {
	    		DBInitializer.load(database);
			} catch (SQLException e) {
				logger.error("Error While connecting to database", e);
				System.exit(1);
			} catch (ClassNotFoundException e) {
				logger.error("Could not connect to database, database communicator JAR is missing");
			}
	    }
		
		try {
			dbAccess = new DBAccess(DriverManager.getConnection(database));
			dbAccess.loadData();
		} catch (SQLException e) {
			logger.error("Error While connecting to database", e);
			System.exit(1);
		} 
    	
    }
    
    /**
     * sets up services for tag swipes
     */
    private static void initIdTagHandlers(){
    	//Handler for idTag updates
    	IdTagHandler idTagHandler = new IdTagHandler(dbAccess);
    	serverThread.addHandler("/dev*", idTagHandler);
    	//Display
    	DisplayName display = new DisplayName("IEEE");
    	idTagHandler.addObserver(display);
    	display.setupDisplay();
    	
    	//Sip presence
    	SipLayer sipLayer;
		try {
			String pName = "presence.properties";
			String location = "not set";
			String ip = "127.0.0.1";
			int port = 8080;
			try {
				Properties props = PropertyLoader.loadProps(pName);
				location = props.getProperty("PHYSICAL_LOCATION");
				ip = props.getProperty("PRESENCE_IP");
				port = new Integer(props.getProperty("PRESENCE_PORT"));
				
			} catch (IOException e) {
				logger.error("No properties file "+ pName + ", using default",e);
				
			} catch (NumberFormatException e){
				logger.error("could not get property PRESENCE_PORT, using default",e);
				
			}
			sipLayer = new SipLayer(ip, port);
			Presence presence = new Presence(sipLayer);
	    	SimplePresenceHandler presenceHandler = new SimplePresenceHandler(sipLayer, presence, location);
	    	idTagHandler.addObserver(presenceHandler);
		} catch (Throwable e) {
			logger.error("Could not initialize sip", e);
		} 
    	
    	//Text Service 
    	TextMarksHandler txtMarks = new TextMarksHandler();
    	idTagHandler.addObserver(txtMarks);

    }
    
    /**
     * inits kiosk mode
     */
    private static void initKiosk(){
    	RegistrationHandler rg = new RegistrationHandler(dbAccess);
    	serverThread.addHandler("/kiosk*", rg);
    }
}
