/*
 *http://svn.apache.org/repos/asf/httpcomponents/httpcore/branches/4.0.x/httpcore/src/examples/org/apache/http/examples/ElementalHttpServer.java
 * ====================================================================
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

package edu.columbia.cs.irt.rfidentify.server;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InterruptedIOException;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.Properties;

import org.apache.http.impl.DefaultConnectionReuseStrategy;
import org.apache.http.impl.DefaultHttpResponseFactory;
import org.apache.http.impl.DefaultHttpServerConnection;
import org.apache.http.params.BasicHttpParams;
import org.apache.http.params.CoreConnectionPNames;
import org.apache.http.params.CoreProtocolPNames;
import org.apache.http.params.HttpParams;
import org.apache.http.protocol.BasicHttpProcessor;
import org.apache.http.protocol.HttpRequestHandler;
import org.apache.http.protocol.HttpRequestHandlerRegistry;
import org.apache.http.protocol.HttpService;
import org.apache.http.protocol.ResponseConnControl;
import org.apache.http.protocol.ResponseContent;
import org.apache.http.protocol.ResponseDate;
import org.apache.http.protocol.ResponseServer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import edu.columbia.cs.irt.rfidentify.core.PropertyLoader;

/**
 * 
 * @author "Blake Arnold"
 * 
 */
public class RequestListenerThread extends Thread {

	private final ServerSocket serversocket;
	private final HttpParams params;
	private final HttpService httpService;
	private final HttpRequestHandlerRegistry reqistry;
	private int port;
	private String docroot;
	private int timeout;
	private final Logger logger = LoggerFactory.getLogger(RequestListenerThread.class);
	private int workers;
	
	public RequestListenerThread()
			throws IOException {
		this.loadProps();
		this.printProps();
		this.serversocket = new ServerSocket(port);
		this.params = new BasicHttpParams();
		this.params.setIntParameter(CoreConnectionPNames.SO_TIMEOUT, timeout)
				.setIntParameter(CoreConnectionPNames.SOCKET_BUFFER_SIZE,
						8 * 1024).setBooleanParameter(
						CoreConnectionPNames.STALE_CONNECTION_CHECK, false)
				.setBooleanParameter(CoreConnectionPNames.TCP_NODELAY, true)
				.setParameter(CoreProtocolPNames.ORIGIN_SERVER,
						"HttpComponents/1.1");

		// Set up the HTTP protocol processor
		BasicHttpProcessor httpproc = new BasicHttpProcessor();
		httpproc.addInterceptor(new ResponseDate());
		httpproc.addInterceptor(new ResponseServer());
		httpproc.addInterceptor(new ResponseContent());
		httpproc.addInterceptor(new ResponseConnControl());

		// Set up request handlers
		this.reqistry = new HttpRequestHandlerRegistry();
		reqistry.register("*", new HttpFileHandler(docroot));
		

		// Set up the HTTP service
		this.httpService = new HttpService(httpproc,
				new DefaultConnectionReuseStrategy(),
				new DefaultHttpResponseFactory());
		this.httpService.setParams(this.params);
		this.httpService.setHandlerResolver(reqistry);
	}
	
	public void addHandler(String pattern, HttpRequestHandler h){
		this.reqistry.register(pattern, h);
		this.httpService.setHandlerResolver(reqistry);
	}

	@Override
	public void run() {
		logger.info("Listening on  " + this.serversocket.getInetAddress() + ":"
				+ this.serversocket.getLocalPort());
		while (!Thread.interrupted()) {
			try {
				// Set up HTTP connection
				Socket socket = this.serversocket.accept();
				DefaultHttpServerConnection conn = new DefaultHttpServerConnection();
				logger.info("Incoming connection from "
						+ socket.getInetAddress());
				conn.bind(socket, this.params);

				// Start worker thread
				Thread t = new WorkerThread(this.httpService, conn);
				t.setDaemon(true);
				t.start();
			} catch (InterruptedIOException ex) {
				break;
			} catch (IOException e) {
				System.err.println("I/O error initialising connection thread: "
						+ e.getMessage());
				break;
			}
		}
	}
	/**
	 * load www-server.properties from java.home 
	 **/
	private void loadProps() {
		String pName = "www-server.properties";
		try {
			Properties props = PropertyLoader.loadProps(pName);
			String p = props.getProperty("port");

			if (p != null) {
				port = new Integer(p);
			} else {
				logger.error("no property \"port\"");
			}

			String r = props.getProperty("root");
			
			
			if (r != null) {
				docroot = r;
				if (!new File(r).exists()) {
					logger.error(docroot + " doesn't exist as server root");
				}
			} else {
				logger.error("no property \"root\"");
				docroot = System.getProperty("user.dir");
			}

			r = props.getProperty("timeout");
			if (r != null) {
				try {
					timeout = Integer.parseInt(r);
				} catch (NumberFormatException e) {
					logger.error("Could not convert " + timeout
							+ "to an number", e);
					timeout = 5000;
				}
			} else {
				logger.error("no property \"timeout\"");
			}

			r = props.getProperty("workers");
			if (r != null) {
				try {
					workers = Integer.parseInt(r);
				} catch (NumberFormatException e) {
					logger.error("Could not convert " + timeout
							+ "to an number", e);
					workers = 5;
				}

			} else {
				logger.error("no property \"workers\"");
			}

			/* sanity check info */

			if (port < 1024 || port > 60000)
				port = 8080;

			if (timeout <= 1000) {
				timeout = 5000;
			}
			if (workers > 25 || workers < 1) {
				workers = 5;
			}

		} catch (FileNotFoundException e) {
			logger.error("File " + pName + " does not exist " + e.getMessage(),
					e);
		} catch (IOException e) {
			logger.error("Error while loading property file " + pName
					+ " Loading defaults" + e.getMessage(), e);
		}

	}

	/**
	 * prints properties
	 */
	void printProps() {

		logger.info("port=" + port);
		logger.info("root=" + docroot);
		logger.info("timeout=" + timeout);
		logger.info("workers=" + workers);
	}

	public int getPort() {
		
		return port;
	}
}